         LIST P=16C54
;****************************************************************
;       3-Wire Byte Write Program (106 bytes)
;
;       This program demonstrates how to interface a  
;       Microchip PIC 16C54 to a 93LC56 or 93LC66 Serial EE 
;       device. This program will execute the erase/write enable
;       command, write to 8 consecutive addresses, and then 
;       execute the erase/write disable command.  This
;       sequence will repeat forever.
;
;       After each byte is written, time must be given to the
;       device for it to complete the write cycle before
;       the next command can be sent.  The easiest solution
;       is to consult the data book for the maximum write
;       cycle time and just wait that long before the next 
;       command is sent.  This program demonstrates that
;       solution.
;
;       Another, more efficient method of determining when the
;       write cycle is complete is called 'data polling.'  This
;       method is demonstrated in the program "3wdpoll."       
;
;       This program communicates to the serial EE in the
;       x16 mode, and ASSUMES THE USER HAS SET THE ORG PIN
;       ON THE DEVICE TO Vcc.  
;
;       Timing is based on using the PIC in 'XT' mode
;       using a 4Mhz crystal.  Clock speeds to the serial EE 
;       will be approximately 40 Khz for this setup.
;
;        PIC to Serial EE Connections:
;
;          PIC            Serial EE
;       ------------     ------------
;       Pin 10 (RB4) --> Chip Select
;       Pin 11 (RB5) --> Clock
;       Pin 12 (RB6) --> Data In
;       Pin 13 (RB7) --> Data Out
;                        ORG=Vcc
;
;************************************************************
;             Register Assignments
;************************************************************
port_a  equ     5h     ; port 5 (port_a)
port_b  equ     6h     ; port 6 (port b) comm lines to serial EE
eeprom  equ    0ah     ; bit buffer
addr    equ    0ch     ; address register
datai   equ    0dh     ; stored data input reg.
datao   equ    0eh     ; stored data output reg.
txbuf   equ    10h     ; transmit buffer
count   equ    11h     ; bits transmitted so far
bits    equ    12h     ; bits to transmit 
bytcnt  equ    13h     ; byte counter for write routine
loops   equ    15h     ; delay loop counter
loops2  equ    16h     ; delay loop counter        ;
;************************************************************
;            Bit Assignments
;************************************************************
di      equ    7       ; eeprom input
do      equ    6       ; eeprom output
datout  equ    7       ; data out line (port_b)
datin   equ    6       ; data in line (port_b) 
sclk    equ    5       ; clock line (port_b)
chpsel  equ    4       ; chip select line (port_b)
;
;*************************************************************
        org    01ffh
begin   goto   PWRUP   ; set the reset vector  
        org    000h
        goto   PWRUP    
;
;*************************************************************
;       DELAY ROUTINE
;       This routine takes the value in 'loops'
;       and multiplies it times 1 millisecond to
;       determine delay time.  
;*************************************************************
WAIT
;
top      movlw   .110           ; timing adjustment variable
         movwf   loops2
top2     nop                    ; sit and wait
         nop
         nop
         nop
         nop
         nop
         decfsz  loops2         ; inner loops complete?
         goto    top2           ; no, go again
                                ;
         decfsz  loops          ; outer loops complete?
         goto    top            ; no, go again
         retlw   0              ; yes, return from sub
;
;**************************************************************
;       Start Bit Subroutine
;       this routine generates a start bit
;       (Chip select and DI high when clock goes high)          
;**************************************************************;
BSTART
         movlw   b'10001111'
         tris    port_b         ; set port b for output
                                ; except for the data out line 
         bcf     port_b,datin   ; set datain and chipselect lines
         bcf     port_b,chpsel  ; low just to check operation
         bcf     port_b,sclk    ; make sure clock starts low too.
         nop
;
         bsf     port_b,chpsel  ; set chip select line high
         bsf     port_b,datin   ; set data in line high
         nop
         bsf     port_b,sclk    ; set the clock line high to
                                ; generate the start bit
         nop
         nop
         bcf     port_b,sclk    ; set clock low again
         retlw   0
         ;
;*************************************************************
;       BITOUT routine 
;       This routine takes one bit of data in 'do' and
;       transmits it to the serial EE device
;*************************************************************
BITOUT
         btfss   eeprom,do      ; check state of data bit
         goto    bitlow         ; low, goto bitlow
         bsf     port_b,datin   ; high, set datain high 
         goto    clkout         ; and clock it                     
                                ;
bitlow   bcf     port_b,datin   ; output a logic low 
clkout   bsf     port_b,sclk    ; set clock line high
         nop
         bcf     port_b,sclk    ; return clock line low
         retlw   0
;                                                              
;****************************************************************
;       Transmit Data Subroutine
;       This routine takes the byte of data stored in the
;       'datao' register and transmits it to the serial EE device.
;****************************************************************
TX
         movf    bits,w         ; set the number of bits to xmit
         movwf   count
         ;
TXLP
         bcf     eeprom,do      ; assume bit 7 is low
         btfsc   txbuf,7        ; is bit 7 clear?
         bsf     eeprom,do      ; no, set data bit =1
         call    BITOUT         ; transmit 1 bit to serial EE
         rlf     txbuf          ; rotate txbuf left
         decfsz  count          ; all bits done?
         goto    TXLP           ; no, do another bit
         retlw   0              ; yes, jump out
;                   
;****************************************************************
;            POWER-UP ROUTINE
;       This is the program entry point, which in this case simply
;       sets the port_a I/O lines and directs control to the 
;       erase/write enable routine.
;*****************************************************************
PWRUP
;
         movlw   b'00000000'
         tris    port_a         ; set port_a as all output
         clrf    port_a         ; all lines low

         movlw   b'10000000'    
         tris    port_b         ; set RB7 as input, rest output;
;
;        Fall through and do erase/write enable
;
;****************************************************************
;          EWEN (Erase/Write ENable Routine)
;            this routine enables the dut for erasing and
;            writing. This must be done prior to any erase,write
;            eral,wral instructions.
;****************************************************************
EWEN
;
         call    BSTART         ; generate a start bit
                                ;
         movlw   .2             ; set # bits to 2
         movwf   bits           ; 
         movlw   b'00000000'    ; get the opcode (00b)   
         movwf   txbuf          ; into the output buffer   
         call    TX             ; and transmit it     
         movlw   .8             ; set # bits to 8
         movwf   bits           ;
         movlw   b'11000000'    ; get opcode and address
                                ;     (11XXXXXX)
         movwf   txbuf          ; into output buffer
         call    TX             ; and transmit it     
         bcf     port_b,chpsel  ; set chip select line low
         nop
;
;        Now continue on to the write command
;
;**************************************************************
;       Byte Write Routine 
;       This routine writes an AA55h pattern into
;       8 consecutive addresses starting at address 00.
;       A delay of about 10ms is given after each byte
;       for the write cycle to complete.
;       The write is done in the x16 mode: the user must
;       have the ORG pin tied to Vcc on the device
;****************************************************************
WRITE
;
         movlw   .0            ; set starting address to 00
         movwf   addr          ; 
         movlw   .8            ; set number of bytes to write as 8
         movwf   bytcnt        ;
                               ;
topwr    call    BSTART        ; generate the start bit
                               ;
         movlw   .2            ; set # bits to 2 for the opcode
         movwf   bits          ;
         movlw   b'01000000'   ; get opcode (01b)
         movwf   txbuf         ; into the transmit buffer
         call    TX            ; and send it
                               ;
         movlw   .8            ; set # of bits to 8 for the
         movwf   bits          ; address 
         movf    addr,w        ; get address counter
         movwf   txbuf         ; into output buffer
         call    TX            ; and send it
         movlw   b'10101010'   ; get upper byte of data (AAh)
         movwf   txbuf         ; into the transmit buffer
         call    TX            ; and send it
         movlw   b'01010101'   ; get lower byte of data (55h)
         movwf   txbuf         ; into transmit buffer
         call    TX            ; and send it
                               ;
         bcf     port_b,chpsel ; clear the chip select line
                               ; to initiate write cycle
                               ;
         movlw   .10           ; 
         movwf   loops         ; set delay time to 10mS
         call    WAIT          ; and wait 
                               ;
         incf    addr          ; increment address counter 
         decfsz  bytcnt        ; all bytes written?
         goto    topwr         ; no, do another
                               ; yes, go on
;
;     Now continue on to the erase/write disable command
;
;****************************************************************
;       EWDS (Erase/Write Disable Routine)
;       This routine executes the erase/write disable command
;       which prevents the contents of the array from being
;       written to.
;
;****************************************************************
EWDS
;
         call    BSTART         ; generate a start bit
;
         movlw   .2             ; set # bits to 2
         movwf   bits           ; 
         movlw   b'00000000'    ; get the opcode (00b)   
         movwf   txbuf          ; into the output buffer   
         call    TX             ; and transmit it     
         movlw   .8             ; set # bits to 8
         movwf   bits           ;
         movlw   b'00000000'    ; get opcode and address
                                ;     (00XXXXXX)
         movwf   txbuf          ; into output buffer
         call    TX             ; and transmit it     
         bcf     port_b,chpsel  ; set chip select line low
         nop
         goto    EWEN           ; start all over
;
END              
