         LIST P=16C54
;****************************************************************
;       3-Wire Byte Write with Data Poll Program (107 bytes)
;
;       This program demonstrates how to interface a  
;       Microchip PIC 16C54 to a 93LC56 or 93LC66 Serial EE 
;       device. This program will execute the erase/write enable
;       command, write to 8 consecutive addresses, and then 
;       use the 'data polling' method to determine when the
;       write cycle is complete.  The program then executes
;       the erase/write disable command.  This sequence will 
;       repeat forever.
;
;       When writing to a 3-wire serial EE device, the
;       internally timed write cycle will begin after 
;       the opcode, address and data are sent to the
;       device.  There are two ways to make sure that the 
;       cycle is complete before you send it another command. 
;       The simplest method is to simply wait for the maximum 
;       write cycle time, which can be obtained from the data book.
;       A more efficient method is "Data polling."  This is
;       done by toggling the chip select line low and then
;       back high after the opcode, address and data are sent.
;       The chip select line must be low for at least 250ns
;       before bringing it high again.  The device will pull
;       the data out line low at that point, and set it high
;       when the write cycle is complete.  The user can then check
;       or "poll" the dataout line until it goes high before
;       sending the next command.       
;
;       As an option, the user can connect a LED to pin 18
;       on the PIC (use about a 1K resistor in series) as a 
;       timeout indicator.  This LED will come on if the
;       data polling is unsuccessful and the device being
;       programmed does not respond.  This can be tested by
;       simply running the program with no part in the socket.
;
;       This program communicates to the serial EE in the
;       x16 mode, and ASSUMES THE USER HAS SET THE ORG PIN
;       ON THE DEVICE TO Vcc.  
;
;       Timing is based on using the PIC in 'XT' mode
;       using a 4Mhz crystal.  Clock speeds to the serial EE 
;       will be approximately 50 Khz for this setup.
;
;        PIC to Serial EE Connections:
;
;          PIC            Serial EE
;       ------------     ------------
;       Pin 10 (RB4) --> Chip Select
;       Pin 11 (RB5) --> Clock
;       Pin 12 (RB6) --> Data In
;       Pin 13 (RB7) --> Data Out
;                        ORG=Vcc
;
;************************************************************
;             Register Assignments
;************************************************************
port_a  equ     5h     ; port 5 (port_a)
port_b  equ     6h     ; port 6 (port b) used comm lines to serial EE
eeprom  equ    0ah     ; bit buffer
addr    equ    0ch     ; address register
datai   equ    0dh     ; stored data input reg.
datao   equ    0eh     ; stored data output reg.
txbuf   equ    10h     ; transmit buffer
count   equ    11h     ; bits transmitted so far
bits    equ    12h     ; bits to transmit 
bytcnt  equ    13h     ; byte counter for write routine
loops   equ    15h     ; delay loop counter
loops2  equ    16h     ; delay loop counter        
;************************************************************
;            Bit Assignments
;************************************************************
di      equ    7       ; eeprom input
do      equ    6       ; eeprom output
datout  equ    7       ; data out line (port_b)
datin   equ    6       ; data in line (port_b) 
sclk    equ    5       ; clock line (port_b)
chpsel  equ    4       ; chip select line (port_b)
timeout equ    1       ; write cycle timeout warning (port_a)
;
;*************************************************************
        org    01ffh
begin   goto   PWRUP   ; set the reset vector  
        org    000h
        goto   PWRUP    
;
;*************************************************************
;       DATA POLL DELAY ROUTINE
;       This routine delays 100 us is
;       used for delay between polls
;*************************************************************
dpdelay
;
         movlw   .25            ; timing adjustment variable
         movwf   loops2
top      nop
         decfsz  loops2         ; loops complete?
         goto    top            ; no, go again
                                ;
         retlw   0              ; yes, return from sub
;
;**************************************************************
;       Start Bit Subroutine
;       this routine generates a start bit
;       (Chip select and DI high when clock goes high)          
;**************************************************************;
BSTART
         bcf     port_b,datin   ; set datain and chipselect lines
         bcf     port_b,chpsel  ; low just to check operation
         bcf     port_b,sclk    ; make sure clock starts low too.
         nop
;
         bsf     port_b,chpsel  ; set chip select line high
         bsf     port_b,datin   ; set data in line high
         nop
         bsf     port_b,sclk    ; set the clock line high to
                                ; generate the start bit
         nop
         nop
         bcf     port_b,sclk    ; set clock low again
         retlw   0
         ;
;*************************************************************
;       BITOUT routine 
;       This routine takes one bit of data in 'do' and
;       transmits it to the serial EE device
;*************************************************************
BITOUT
         btfss   eeprom,do      ; check state of data bit
         goto    bitlow         ; low, goto bitlow
         bsf     port_b,datin   ; high, set datain high 
         goto    clkout         ; and clock it                     
                                ;
bitlow   bcf     port_b,datin   ; output a logic low 
clkout   bsf     port_b,sclk    ; set clock line high
         nop
         bcf     port_b,sclk    ; return clock line low
         retlw   0
;                                                              
;****************************************************************
;       Transmit Data Subroutine
;       This routine takes the byte of data stored in the
;       'datao' register and transmits it to the serial EE device.
;****************************************************************
TX
         movf    bits,w         ; set the number of bits to xmit
         movwf   count
         ;
TXLP
         bcf     eeprom,do      ; assume bit 7 is low
         btfsc   txbuf,7        ; is bit 7 clear?
         bsf     eeprom,do      ; no, set data bit =1
         call    BITOUT         ; transmit 1 bit to serial EE
         rlf     txbuf          ; rotate txbuf left
         decfsz  count          ; all bits done?
         goto    TXLP           ; no, do another bit
         retlw   0              ; yes, jump out
;                   
;****************************************************************
;            POWER-UP ROUTINE
;       This is the program entry point, which in this case simply
;       sets the port_a I/O lines and directs control to the 
;       erase/write enable routine.
;*****************************************************************
PWRUP
;
         movlw   b'00000000'
         tris    port_a         ; set port_a as all output
         clrf    port_a         ; all lines low

         movlw   b'10000000'    
         tris    port_b         ; set RB7 as input, rest output;
;
;        Fall through and do erase/write enable
;
;****************************************************************
;          EWEN (Erase/Write ENable Routine)
;            this routine enables the dut for erasing and
;            writing. This must be done prior to any erase,write
;            eral,wral instructions.
;****************************************************************
EWEN
;
         call    BSTART         ; generate a start bit
                                ;
         movlw   .2             ; set # bits to 2
         movwf   bits           ; 
         movlw   b'00000000'    ; get the opcode (00b)   
         movwf   txbuf          ; into the output buffer   
         call    TX             ; and transmit it     
         movlw   .8             ; set # bits to 8
         movwf   bits           ;
         movlw   b'11000000'    ; get opcode and address
                                ;     (11XXXXXX)
         movwf   txbuf          ; into output buffer
         call    TX             ; and transmit it     
         bcf     port_b,chpsel  ; set chip select line low
         nop
;
;        Now continue on to the write command
;
;**************************************************************
;       WRITE 
;       This routine writes a AA55h pattern into
;       8 consecutive addresses starting at address 00.
;       It will then 'poll' the data out line to determine
;       when the write cycle is complete.  If the cycle has
;       not completed within 20 ms, then it will continue
;       anyway and turn the timeout fail LED on.
;       The write is done in the x16 mode: the user must
;       have the ORG pin tied to Vcc on the device. This
;       program will repeat forever.
;****************************************************************
WRITE
;
         movlw   .0            ; set starting address to 00
         movwf   addr          ; 
         movlw   .8            ; set number of bytes to write as 8
         movwf   bytcnt        ;
                               ;
topwr    call    BSTART        ; generate the start bit
                               ;
         movlw   .2            ; set # bits to 2 for the opcode
         movwf   bits          ;
         movlw   b'01000000'   ; get opcode (01b)
         movwf   txbuf         ; into the transmit buffer
         call    TX            ; and send it
                               ;
         movlw   .8            ; set # of bits to 8 for the
         movwf   bits          ; address 
         movf    addr,w        ; get address counter
         movwf   txbuf         ; into output buffer
         call    TX            ; and send it
         movlw   b'10101010'   ; get first half of data (AAh)
         movwf   txbuf         ; into the transmit buffer
         call    TX            ; and send it
         movlw   b'01010101'   ; get second half of data (55h)
         movwf   txbuf         ; into transmit buffer
         call    TX            ; and send it
                               ;
         bcf     port_b,chpsel ; clear the chip select line
                               ; to initiate write cycle
         nop
         bsf     port_b,chpsel ; set the chip sel line high
                               ; to begin data polling
         movlw   .200          ; 
         movwf   loops         ; poll 100 times before timeout(about 20ms) 
polltop  call    dpdelay       ; wait 100us
         btfsc   port_b,datout ; is the data out line high?
         goto    okpoll        ; yes-cycle complete: do another
         decfsz  loops         ; has it timed out?
         goto    polltop       ; no, check again
         bsf     port_a,timeout; yes, set timeout LED and go on
         goto    badpoll                      ;

okpoll   bcf     port_a,timeout; clear the timeout LED
badpoll  bcf     port_b,chpsel ; chip sel line back low
         incf    addr          ; increment address counter 
         decfsz  bytcnt        ; all bytes written?
         goto    topwr         ; no, do another
                               ; yes, go on
;
;     Now continue on to the erase/write disable command
;
;****************************************************************
;          EWDS (Erase/Write Disable Routine)
;       This routine executes the erase/write disable command
;       which prevents the contents of the array from being
;       written to.
;
;****************************************************************
EWDS
;
         call    BSTART         ; generate a start bit
;
         movlw   .2             ; set # bits to 2
         movwf   bits           ; 
         movlw   b'00000000'    ; get the opcode (00b)   
         movwf   txbuf          ; into the output buffer   
         call    TX             ; and transmit it     
         movlw   .8             ; set # bits to 8
         movwf   bits           ;
         movlw   b'00000000'    ; get opcode and address
                                ;     (00XXXXXX)
         movwf   txbuf          ; into output buffer
         call    TX             ; and transmit it     
         bcf     port_b,chpsel  ; set chip select line low
         nop
         goto    EWEN           ; start all over
;
;  

END              
