         LIST P=16C54
;****************************************************************
;       3-Wire Sequential Read Program (93 bytes)
;
;       This program demonstrates how to interface a  
;       Microchip PIC 16C54 to a 93LC56 or 93LC66 Serial EE 
;       device.  This program will read 8 consecutive addresses
;       in the 'sequential read' mode.  This means that the opcode
;       and address are sent for the first address only.  After
;       the first address is read, the Chip Select line is left
;       high and clocks for the remaining 7 bytes are sent to the
;       device. The device will automatically increment the address
;       pointer in this mode, allowing the user to read as many 
;       consecutive addresses as needed. This program will repeat 
;       forever.
;;
;       This program communicates to the serial EE in the
;       x16 mode, and ASSUMES THE USER HAS SET THE ORG PIN
;       ON THE DEVICE TO Vcc.  
;
;       Timing is based on using the PIC in 'XT' mode
;       using a 4Mhz crystal.  Clock speeds to the serial EE 
;       will be approximately 50 Khz for this setup.
;
;        PIC to Serial EE Connections:
;
;          PIC            Serial EE
;       ------------     ------------
;       Pin 10 (RB4) --> Chip Select
;       Pin 11 (RB5) --> Clock
;       Pin 12 (RB6) --> Data In
;       Pin 13 (RB7) --> Data Out
;                        ORG = Vcc
; 
;************************************************************
;             Register Assignments
;************************************************************
status  equ     3h     ; status register
port_a  equ     5h     ; port 5 (port_a)
port_b  equ     6h     ; port 6 (port b) used comm lines to serial EE
eeprom  equ    0ah     ; bit buffer
addr    equ    0ch     ; address register
datai   equ    0dh     ; stored data input reg.
datao   equ    0eh     ; stored data output reg.
txbuf   equ    10h     ; transmit buffer
count   equ    11h     ; bits transmitted so far
bits    equ    12h     ; bits to transmit 
bytcnt  equ    13h     ; byte counter for read routine
loops   equ    15h     ; delay loop counter
loops2  equ    16h     ; delay loop counter        
hbyte   equ    17h     ; high byte for input data
lbyte   equ    18h     ; low byte for input data
;************************************************************
;            Bit Assignments
;************************************************************
di      equ    7       ; eeprom input
do      equ    6       ; eeprom output
datout  equ    7       ; data out line (port_b)
datin   equ    6       ; data in line (port_b) 
sclk    equ    5       ; clock line (port_b)
chpsel  equ    4       ; chip select line (port_b)
;
;*************************************************************
        org    01ffh
begin   goto   PWRUP   ; set the reset vector  
        org    000h
        goto   PWRUP    
;
;*************************************************************
;       DELAY ROUTINE
;       This routine takes the value in 'loops'
;       and multiplies it times 1 millisecond to
;       determine delay time.  
;*************************************************************
WAIT
;
top      movlw   .110           ; timing adjustment variable
         movwf   loops2
top2     nop                    ; sit and wait
         nop
         nop
         nop
         nop
         nop
         decfsz  loops2         ; inner loops complete?
         goto    top2           ; no, go again
                                ;
         decfsz  loops          ; outer loops complete?
         goto    top            ; no, go again
         retlw   0              ; yes, return from sub
;
;**************************************************************
;       Start Bit Subroutine
;       this routine generates a start bit
;       (Chip select and DI high when clock goes high)          
;**************************************************************
BSTART
         bcf     port_b,datin   ; set datain and chipselect lines
         bcf     port_b,chpsel  ; low just to check operation
         bcf     port_b,sclk    ; make sure clock starts low too.
         nop
;
         bsf     port_b,chpsel  ; set chip select line high
         bsf     port_b,datin   ; set data in line high
         nop
         bsf     port_b,sclk    ; set the clock line high to
                                ; generate the start bit
         nop
         nop
         bcf     port_b,sclk    ; set clock low again
         retlw   0
         ;
;**************************************************************
;       BITIN routine reads one bit of data from the 
;       serial EE device and stores it in 'di'
;**************************************************************
BITIN
         bsf     eeprom,di      ; assume input bit is high
         bsf     port_b,sclk    ; set clock line high
         nop                    ; 
         btfss   port_b,datout  ; read the data bit
         bcf     eeprom,di      ; input bit was low
         bcf     port_b,sclk    ; set clock line low
                                ;
         retlw   0              ; 

;****************************************************************
;       Receive data routine
;       This routine reads one byte of data from the part
;       into the 'datai' register.
;****************************************************************
RX
         clrf    datai          ; clear input buffer
         movlw   .8             ; set # bits to 8
         movwf   count      
         bcf     status,0       ; make sure carry bit is low
RXLP     rlf     datai          ; rotate the buffer left 1 bit
         call    BITIN          ; read 1 bit
         bcf     datai,0        ; assume the input bit was low
         btfsc   eeprom,di      ; check the bit 
         bsf     datai,0        ; set high if neccessary
         decfsz  count          ; 8 bits done?
         goto    RXLP           ; no, do another
         retlw   0
;    
;*************************************************************
;       BITOUT routine 
;       This routine takes one bit of data in 'do' and
;       transmits it to the serial EE device
;*************************************************************
BITOUT
         btfss   eeprom,do      ; check state of data bit
         goto    bitlow         ; low, goto bitlow
         bsf     port_b,datin   ; high, set datain high 
         goto    clkout         ; and clock it                     
                                ;
bitlow   bcf     port_b,datin   ; output a logic low 
clkout   bsf     port_b,sclk    ; set clock line high
         nop
         bcf     port_b,sclk    ; return clock line low
         retlw   0
;                                                              
;****************************************************************
;       Transmit Data Subroutine
;       This routine takes the byte of data stored in the
;       'datao' register and transmits it to the serial EE device.
;****************************************************************
TX
         movf    bits,w         ; set the number of bits to xmit
         movwf   count
         ;
TXLP
         bcf     eeprom,do      ; assume bit 7 is low
         btfsc   txbuf,7        ; is bit 7 clear?
         bsf     eeprom,do      ; no, set data bit =1
         call    BITOUT         ; transmit 1 bit to serial EE
         rlf     txbuf          ; rotate txbuf left
         decfsz  count          ; all bits done?
         goto    TXLP           ; no, do another bit
         retlw   0              ; yes, jump out
;                   
;****************************************************************
;            POWER-UP ROUTINE
;       This is the program entry point, which in this case simply
;       sets the port_a I/O lines and directs control to the 
;       erase/write enable routine.
;*****************************************************************
PWRUP
;
         movlw   b'00010001'
         tris    port_a         ; set RA0 as input, rest output
         clrf    port_a         ; all lines low
         movlw   b'10000000'    
         tris    port_b         ; set RB7 as input, rest output;
;
;               Fall through and do the read
;
;********************************************************************* 
;       READ ROUTINE (Sequential read mode)
;       This routine reads 8 consecutive addresses in 
;       sequential mode starting at address 0.  This 
;       program will repeat forever               
;*********************************************************************
READ
;
        movlw   .0             ; set starting address to 00    
        movwf   addr           ;          
        movlw   .8             ; set number of addresses to
        movwf   bytcnt         ; read as 8 
                               ;
        call    BSTART         ; generate the start bit
        movlw   .2             ; set # bits to 2
        movwf   bits           ; 
        movlw   b'10000000'    ; get opcode (10b)
        movwf   txbuf          ; into output buffer
        call    TX             ; and transmit it
        movlw   .8             ; 
        movwf   bits           ; set number of bits to 8
        movf    addr,w         ; get the address
        movwf   txbuf          ; into the output buffer
        call    TX             ; and transmit it

rbyte   call    RX             ; read the high byte
        movf    datai,w        ; move input data to w
        movwf   hbyte          ; xfer it to high byte 

        call    RX             ; read the low byte
        movf    datai,w        ; move input data to w
        movwf   hbyte          ; xfer it to low byte 
                                
        incf    addr           ; add 1 to the address
        decfsz  bytcnt         ; have all bytes been read?
        goto    rbyte          ; no, read another byte
   
        bcf     port_b,chpsel  ; yes,clear the chip select line
        goto    READ           ; and start over
END              
